@@ -5,7 +5,7 @@ use Mouse;
 
 #use Lemonldap::NG::Handler::Main qw(:jailSharedVars);
 
-our $VERSION = '2.17.0';
+our $VERSION = '2.19.0';
 
 has protection => ( is => 'rw', isa => 'Str' );
 has rule       => ( is => 'rw', isa => 'Str' );
@@ -28,6 +28,7 @@ sub init {
         );
         return 0;
     }
+
     eval { $self->portal( $self->api->tsv->{portal}->() ) };
     my $rule =
       $args->{protection} || $self->api->localConfig->{protection} || '';
@@ -135,7 +136,7 @@ sub _authAndTrace {
     eval "require $type";
     die $@ if ($@);
     my ( $res, $session ) = $type->run( $req, $self->{rule} );
-    eval { $self->portal( $type->tsv->{portal}->() ) } unless $self->portal;
+    my $portal = eval { $type->tsv->{portal}->($req) };
     $self->logger->warn($@)  if $@;
     $req->userData($session) if ($session);
 
@@ -155,18 +156,16 @@ sub _authAndTrace {
             my %h    = ( $req->spliceHdrs );
             my $host = $req->env->{HTTP_HOST};
             if (    $h{Location}
-                and $h{Location} =~ m#^\Q$self->{portal}\E#
+                and $h{Location} =~ m#^\Q$portal\E#
                 and $h{Location} !~ m#^https?://$host# )
             {
-                return [
-                    401, [ 'WWW-Authenticate' => 'SSO ' . $self->{portal} ], []
-                ];
+                return [ 401, [ 'WWW-Authenticate' => 'SSO ' . $portal ], [] ];
             }
         }
         return [ $res, [ $req->spliceHdrs ], [] ];
     }
     else {
-        my $s = ( $self->portal ? $self->portal . "/lmerror/$res" : '' );
+        my $s = ( $portal ? $portal . "/lmerror/$res" : '' );
         $s =
             '<html><head><title>Redirection</title></head><body>'
           . qq{<script type="text/javascript">window.location='$s'</script>}
@@ -232,7 +231,7 @@ sub group {
 sub sendError {
     my ( $self, $req, $err, $code ) = @_;
     $err ||= $req->error;
-    $self->userLogger->warn( '[' . $self->userId($req) . "] $err" );
+    $self->logger->warn( '[' . $self->userId($req) . "] $err" );
     return $self->Lemonldap::NG::Common::PSGI::sendError( $req, $err, $code );
 }
 
