@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Init;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.19.0';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -50,7 +50,8 @@ sub init($$) {
 # Set log level for Lemonldap::NG logs
 sub logLevelInit {
     my ($class) = @_;
-    my $logger  = $class->localConfig->{logger} ||= $class->defaultLogger;
+    my $logger = $class->localConfig->{logger} ||=
+      $ENV{LLNG_DEFAULTLOGGER} || $class->defaultLogger;
     eval "require $logger";
     die $@ if ($@);
     my $err;
@@ -79,6 +80,14 @@ sub logLevelInit {
         )
     );
     $class->logger->debug("User logger $logger loaded");
+
+    my $auditlogger =
+         $ENV{LLNG_AUDITLOGGER}
+      || $class->localConfig->{auditLogger}
+      || "Lemonldap::NG::Common::AuditLogger::UserLoggerCompat";
+    eval "require $auditlogger";
+    die $@ if ($@);
+    $class->_auditLogger( $auditlogger->new($class) );
 }
 
 # @method void serverSignatureInit
